/*
 * Copyright (C) 2019 Alexander Borisov
 *
 * Author: Alexander Borisov <borisov@lexbor.com>
 */

#include "lexbor/encoding/encoding.h"
#include "lexbor/encoding/multi.h"


#define append_to_file(fc, data, len, codepoint)                               \
    do {                                                                       \
        lxb_char_t *data_to = data;                                            \
                                                                               \
        for (size_t j = 0; j < len; j++) {                                     \
            fprintf(fc, "\\x%02X", (unsigned) data_to[j]);                     \
        }                                                                      \
                                                                               \
        fprintf(fc, " 0x%04X\n", (unsigned) codepoint);                        \
    }                                                                          \
    while (0)

#define encode_to_file(fc, _cp)                                                \
    do {                                                                       \
        cps = &_cp;                                                            \
                                                                               \
        lxb_encoding_encode_init(&ctx, enc_data, data, sizeof(data));          \
                                                                               \
        status = enc_data->encode(&ctx, &cps, (cps + 1));                      \
        if (status != LXB_STATUS_OK) {                                         \
            printf("Failed to encoding code point: %u\n", _cp);                \
            return EXIT_FAILURE;                                               \
        }                                                                      \
                                                                               \
        if (*data > 0x9F) {                                                    \
            break;                                                             \
        }                                                                      \
                                                                               \
        append_to_file(fc, ctx.buffer_out, ctx.buffer_used, _cp);              \
    }                                                                          \
    while (0)


int main(int argc, const char * argv[])
{
    size_t size;
    lxb_char_t data[8];
    lxb_status_t status;
    lxb_encoding_encode_t ctx;
    const lxb_codepoint_t *cps;
    const lxb_encoding_data_t *enc_data;
    const lxb_encoding_multi_index_t *entry;

    const char *filepath = "./shift_jis_map_decode.txt";

    enc_data = lxb_encoding_data(LXB_ENCODING_SHIFT_JIS);

    FILE *fc = fopen(filepath, "w");
    if (fc == NULL) {
        printf("Failed to opening file: %s\n", filepath);
        exit(EXIT_FAILURE);
    }

    fprintf(fc, "#\n"
            "# Copyright (C) 2019 Alexander Borisov\n"
            "#\n"
            "# Author: Alexander Borisov <borisov@lexbor.com>\n"
            "#\n\n");

    fprintf(fc, "#\n"
            "# This file generated by the program\n"
            "# \"utils/lexbor/encoding/shift_jis_map_decode.c\"\n"
            "#\n\n");

    /* Single index */
    size = sizeof(lxb_encoding_multi_index_jis0208)
           / sizeof(lxb_encoding_multi_index_t);

    for (size_t i = 0; i < size; i++) {
        entry = &lxb_encoding_multi_index_jis0208[i];

        if (entry->codepoint == LXB_ENCODING_ERROR_CODEPOINT) {
            continue;
        }

        encode_to_file(fc, entry->codepoint);
    }

    for (lxb_codepoint_t i = 0xFF61; i <= 0xFF9F; i++) {
        encode_to_file(fc, i);
    }

    for (lxb_codepoint_t i = 0x00; i <= 0x80; i++) {
        if (i != 0x5C && i != 0x7E) {
            encode_to_file(fc, i);
        }
    }

    fprintf(fc, "\n# END\n");

    fclose(fc);

    return EXIT_SUCCESS;
}
